﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.Binary;

namespace HeuristicLab.Networks.FeatureSelection_Network {
  public class SelectionProblem : BinaryProblem {
    private Func<BinaryVector, double> eval;

    protected SelectionProblem(SelectionProblem original, Cloner cloner)
      : base(original, cloner) {
    }
    [StorableConstructor]
    protected SelectionProblem(bool deserializing) : base(deserializing) { }

    public SelectionProblem(Func<BinaryVector, double> eval, bool maximization)
      : base() {
      this.maximization = maximization;
      this.eval = eval;


      var lengthParam = new ValueParameter<IntValue>("Length");


      lengthParam.ValueChanged += (o, s) => { Length = lengthParam.Value.Value; };
      // we cannot use the fixed value parameter from BinaryProblem in the network
      Parameters.Remove("Length");
      Parameters.Add(lengthParam);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SelectionProblem(this, cloner);
    }

    public override double Evaluate(BinaryVector vector, IRandom random) {
      return eval(vector);

    }

    private readonly bool maximization;
    public override bool Maximization {
      get { return maximization; }
    }
  }
}
